(function (window, document) {
  var CE_GMNS = {};

  CE_GMNS.currentMessageView = null;
  CE_GMNS.currentMessageViewId = null;
  CE_GMNS.expandedMessages = [];
  CE_GMNS.sentMessages = {};

  CE_GMNS_APP_ORIGIN = "https://gmns.cloudextend.app";

  InboxSDK.load(2, "sdk_cega-gmail_a00281a1f8").then(function (sdk) {
    appIframe = document.createElement("iframe");
    appIframe.setAttribute("src", CE_GMNS_APP_ORIGIN);
    appIframe.setAttribute("class", "celigo-app-iframe");
    appIframe.setAttribute("name", "cegaApp");

    sdk.Global.addSidebarContentPanel({
      iconUrl: chrome.extension.getURL("assets/Icon48.png"),
      hideTitleBar: true,
      el: appIframe,
    }).then((ContentPanelView) => {
      sdk.Conversations.registerMessageViewHandler(function (messageView) {
        messageView.isLoaded()
          ? initMessageView(messageView, sdk)
          : messageView.on("load", function () {
              return initMessageView(messageView, sdk);
            });

        sdk.User.isConversationViewDisabled().then(
          (isConversationViewDisabled) => {
            if (!isConversationViewDisabled) {
              var contentSpan = document.createElement("span");
              contentSpan.className = "celigo-gmns-span";

              var navigate = chrome.extension.getURL("assets/navigate.png");
              var textContent = document.createElement("span");

              textContent.innerHTML =
                'Open with CloudExtend <img class="navigate" src="' +
                navigate +
                '" alt=">">';

              textContent.addEventListener("click", function () {
                if (!ContentPanelView.isActive()) {
                  ContentPanelView.open();
                }
                onMessageLoaded(messageView, sdk);
              });

              contentSpan.appendChild(textContent);
              messageView
                .getBodyElement()
                .insertBefore(
                  contentSpan,
                  messageView.getBodyElement().firstElementChild
                );
            }
          }
        );
      });
    });

    window.addEventListener(
      "message",
      (event) => {
        if (event.data.type === "SdkEventListnerReady") {
          console.log(
            "[InboxSdk Loader] event received: " + JSON.stringify(event.data)
          );

          if (
            event.data.user.toLowerCase() !==
            sdk.User.getEmailAddress().toLowerCase()
          ) {
            message = {};
            message.type = "licenseMismatch";
            message.sdkUserEmail = sdk.User.getEmailAddress().toLowerCase();
            console.log(
              "[InboxSdk Loader] : licenseMismatch SDK user: ",
              sdk.User.getEmailAddress()
            );
            postMessage(message);
            return;
          }

          if (CE_GMNS.currentMessageView) {
            generateMessage(
              "messageRead",
              CE_GMNS.currentMessageViewId,
              CE_GMNS.currentMessageView
            ).then((res) => {
              postMessage(res);
            });
          } else {
            message = {};
            message.type = "noContext";
            console.log("[InboxSdk Loader] LIST: ListViewLoaded ");
            postMessage(message);
          }
        }
      },
      false
    );

    sdk.Compose.registerComposeViewHandler(function (composeView) {
      composeView.on("sent", function (event) {
        return onMessageSent(composeView, event);
      });
    });
  });

  function initMessageView(messageView, sdk) {
    //clear sent messages
    for (id in CE_GMNS.sentMessages) {
      if (Date.now() - CE_GMNS.sentMessages[id] > 30000) {
        delete CE_GMNS.sentMessages[id];
      }
    }

    messageView.on("load", function () {
      console.log("[InboxSdk Loader] View: load");
      return onMessageLoaded(messageView, sdk);
    });

    messageView.on("viewStateChange", function (event) {
      console.log("[InboxSdk Loader] View: viewStateChange");
      if (event.newViewState === "EXPANDED") {
        return onMessageLoaded(messageView, sdk);
      }
    });

    messageView.getMessageIDAsync().then(function (messageId) {
      if (
        messageView.isLoaded() &&
        !CE_GMNS.currentMessageView &&
        CE_GMNS.sentMessages[messageId]
      ) {
        onMessageLoaded(messageView, sdk);
      }
    });

    messageView.on("destroy", function () {
      message = {};
      message.type = "messageDestroy";
      console.log("[InboxSdk Loader] MESSAGE: messageDestroy ");
      CE_GMNS.currentMessageView = null;
      CE_GMNS.currentMessageViewId = null;
      CE_GMNS.expandedMessages = [];
      postMessage(message);
    });
  }

  function onMessageLoaded(msgView, sdk) {
    if (msgView === CE_GMNS.currentMessageView) {
      console.log("[InboxSdk Loader] No update required in the App");
      return;
    }
    return msgView.getMessageIDAsync().then(function (messageId) {
      console.log("[InboxSdk Loader] MESSAGE: messageRead ");

      sdk.User.isConversationViewDisabled().then(
        (isConversationViewDisabled) => {
          if (!isConversationViewDisabled) {
            updateCeligoIcon(messageId);
            if (
              msgView.getViewState() === "EXPANDED" &&
              !CE_GMNS.expandedMessages.includes(messageId)
            ) {
              msgView.addAttachmentIcon({
                iconUrl: chrome.extension.getURL("assets/Icon21.png"),
                iconClass: "celigo-icon ce-" + messageId,
                onClick: function () {
                  onMessageLoaded(msgView, sdk);
                },
              });
            }
          }
        }
      );

      CE_GMNS.currentMessageView = msgView;
      CE_GMNS.currentMessageViewId = messageId;
      generateMessage("messageRead", messageId, msgView).then(function (res) {
        CE_GMNS.expandedMessages.push(messageId);
        postMessage(res);
      });
    });
  }

  function updateCeligoIcon(activeMessageId) {
    var elements = document.querySelectorAll(".celigo-icon");
    for (var element of elements) {
      if (!element.classList.contains("ce-" + activeMessageId)) {
        element.style.display = "none";
      } else {
        element.style.display = "inline";
      }
    }
  }

  function onRecipientsChanged(composeView) {
    return composeView.getDraftID().then(function (draftId) {
      console.log("[InboxSdk Loader] COMPOSE: recipientsChanged ");
      generateMessage("recipientsChanged", draftId, composeView).then(function (
        res
      ) {
        postMessage(res);
      });
    });
  }

  function onMessageSent(composeView, event) {
    return event.getMessageID().then(function (messageId) {
      CE_GMNS.sentMessages[messageId] = Date.now();
    });
  }

  function onComposeViewRestored(composeView) {
    return composeView.getDraftID().then(function (draftId) {
      console.log("[InboxSdk Loader] COMPOSE: messageComposeRestored ");
      postMessage(
        generateMessage("messageComposeRestored", draftId, composeView)
      );
    });
  }

  function onComposeViewUnfocused() {
    message = {};
    message.type = "composeViewUnfocused";
    console.log("[InboxSdk Loader] COMPOSE: composeViewUnfocused ");
    postMessage(message);
  }

  function postMessage(message) {
    if (message && window.frames && window.frames.cegaApp) {
      try {
        window.frames.cegaApp.postMessage(
          JSON.stringify(message),
          CE_GMNS_APP_ORIGIN
        );
      } catch (error) {
        console.log(
          "[InboxSdk Loader] postMessage: Error during posting message " +
            JSON.stringify(message)
        );
      }
    }
  }

  function transformContacts(contacts) {
    return contacts.map(function (contact) {
      return { name: contact.name, email: contact.emailAddress };
    });
  }

  //Removing custom banner element prior pushing message to the app
  function filterMailContent(messageElement) {
    var messageClone = messageElement.cloneNode(true);
    var bannerElement =
      messageClone.getElementsByClassName("celigo-gmns-span")[0];
    if (bannerElement) {
      messageClone.removeChild(bannerElement);
    }
    return messageClone.innerHTML;
  }

  function generateMessage(eventType, id, view) {
    message = { id: id, type: eventType };
    switch (eventType) {
      case "messageRead":
        return view.getRecipientsFull().then(function (recipients) {
          var threadView = view.getThreadView();
          return threadView.getThreadIDAsync().then(function (threadId) {
            message.threadId = threadId;
            message.subject = view.getThreadView().getSubject();
            message.body = filterMailContent(view.getBodyElement());
            message.sender = transformContacts([view.getSender()])[0];
            message.recipientsTo = transformContacts(recipients);
            message.timestamp = new Date(view.getDateString()).toISOString();
            var attachments = [];
            view.getFileAttachmentCardViews().filter(function (item) {
              attachments.push({
                id: "",
                name: item.getTitle(),
                attachmentType: item.getAttachmentType(),
              });
            });
            message.attachments = attachments;
            return message;
          });
        });
      case "recipientsChanged":
        return view.getRecipientsFull().then(function (recipients) {
          message.sender = view.getFromContact();
          message.recipientsTo = transformContacts(recipients);
          return message;
        });
      case "messageComposeRestored":
      case "sent":
        return view.getRecipientsFull().then(function (recipients) {
          message.subject = view.getSubject();
          message.body = view.getBodyElement().innerHTML;
          message.sender = transformContacts([view.getFromContact()])[0];
          message.recipientsTo = transformContacts(recipients);
          return message;
        });
      default:
    }
  }
})(window, document);
